/*  dialogs.c -	Dialogs
 *
 *  Stones of Asterind
 *
 *  Matthew J. Gelhaus
 *  October 27, 1998
 */

#include <unistd.h>
#include <gtk/gtk.h>
#include "globals.h"
#include "menu.h"
#include "gameboard.h"
#include "hiscores.h"
#include "dialogs.h"
#include "gtkled.h"


GtkWidget	*wDialog;
GtkWidget	*wHighScoreName;
int		iResponse[6];

struct strOptions
{
  int	iRows,
  	iCols,
  	iColors,
  	bAllowRepeatColors;
};

struct strOptions	Options;


/*
 *  Function to update the high scores list
 */
void UpdateHighScoresList()
{
  char *sName;

  if (!wHighScoreName) return;
  
  /*
   *  Read the player's name from the dialog
   */
  sName = gtk_entry_get_text(GTK_ENTRY(wHighScoreName));
  
  /*
   *  Save the high score entry into the list
   */
  NewHighScore(iScore, sName, uiGameTime, 0, iCurrentRow,
               iNumRows, iNumCols, iNumColors);
}


/*
 *  Button callback - return user response
 */
void DialogResponse(GtkWidget *wButton, gpointer pData)
{
  int *iResponse;

  
  iResponse = (int *)pData;
  
  bResponse = *iResponse;
  
  /*
   *  Update the high score entry, if available
   */
  if (wHighScoreName)
  {
    UpdateHighScoresList();
    wHighScoreName = NULL;
  }

  /*
   *  Close the dialog
   */
  gtk_grab_remove(wDialog);
  gtk_widget_destroy(wDialog);
  
  /*
   *  Start a new game if user answered 'Yes'
   */
  switch (bResponse)
  {
    case RESPONSE_YES:  NewGame(NULL);
                        break;
    case RESPONSE_OK:  iNumRows = Options.iRows;
                       iNumCols = Options.iCols;
                       iNumColors = Options.iColors;
                       bAllowRepeatColors = Options.bAllowRepeatColors;
                       break;
    case RESPONSE_OK_NEW_GAME:  iNumRows = Options.iRows;
                                iNumCols = Options.iCols;
                                iNumColors = Options.iColors;
                                bAllowRepeatColors = Options.bAllowRepeatColors;
                                NewGame(NULL);
                                break;
  }
}


/*
 * Lose Game dialog
 *   Displays solution and prompts for another game
 */
void LoseGameDialog()
{
  GtkWidget	*wLabel,						/* text label					*/
  		*wGrid, *wBox1, *wBox2,			/* packing widgets				*/
  		*wLED,							/* LED widget					*/
  		*wbYes, *wbNo;					/* yes, no buttons				*/
  unsigned int	uiHours, uiMinutes, uiSeconds,	/* playing time					*/
  		uiTenths;
  char		sMessage[255];					/* text message					*/
  int		iCol;							/* column index					*/
  
  /*
   *  Compute the playing time
   */
  uiHours   = uiGameTime/36000;
  uiMinutes = (uiGameTime -uiHours*36000)/600;
  uiSeconds = (uiGameTime -uiHours*36000 -uiMinutes*600)/10;
  uiTenths  = uiGameTime % 10;

  
  wDialog = gtk_window_new(GTK_WINDOW_DIALOG);
  gtk_window_set_title(GTK_WINDOW(wDialog), "Game Over");
  gtk_window_position(GTK_WINDOW(wDialog), GTK_WIN_POS_CENTER);
  gtk_container_border_width(GTK_CONTAINER(wDialog), 10);
  
  wBox1 = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(wDialog), wBox1);
  
  /*
   *  Determine the end-of-game message
   */
  wLabel = gtk_label_new("Sorry, you've lost!");
  gtk_box_pack_start(GTK_BOX(wBox1), wLabel, FALSE, FALSE, 0);
  gtk_widget_show(wLabel);
  
  sprintf(sMessage, "Your time was %02u:%02u:%02u.%1u", uiHours, uiMinutes, uiSeconds,
          uiTenths);
  wLabel = gtk_label_new(sMessage);
  gtk_box_pack_start(GTK_BOX(wBox1), wLabel, FALSE, FALSE, 5);
  gtk_widget_show(wLabel);
  
  /*
   *  Solution
   */
  wLabel = gtk_label_new("Here is the solution:");
  gtk_box_pack_start(GTK_BOX(wBox1), wLabel, FALSE, FALSE, 0);
  gtk_widget_show(wLabel);
  
  wGrid = gtk_table_new(1, iNumCols, TRUE);
  gtk_table_set_col_spacings(GTK_TABLE(wGrid), 2);
  gtk_box_pack_start(GTK_BOX(wBox1), wGrid, FALSE, FALSE, 2);

  for (iCol = 0; iCol < iNumCols; iCol++)
  {
    wLED = gtk_led_new();
    SetButtonColor(wLED, iSolution[iCol]);
    gtk_table_attach(GTK_TABLE(wGrid), wLED,
                     iCol, iCol +1, 0, 1,
                     GTK_FILL | GTK_EXPAND | GTK_SHRINK, 
                     GTK_FILL | GTK_EXPAND | GTK_SHRINK,
                     0, 0);
    gtk_widget_show(wLED);
  }
  gtk_widget_set_usize(wGrid, 100, 16);
  gtk_widget_show(wGrid);
  
  /*
   *  Yes, No buttons
   */
  wLabel = gtk_label_new("Do you want to play again?");
  gtk_box_pack_start(GTK_BOX(wBox1), wLabel, FALSE, FALSE, 5);
  gtk_widget_show(wLabel);
  
  wBox2 = gtk_hbox_new(TRUE, 2);
  gtk_box_pack_end(GTK_BOX(wBox1), wBox2, FALSE, FALSE, 0);
  
  wbYes = gtk_button_new_with_label("Yes");
  iResponse[0] = RESPONSE_YES;
  gtk_signal_connect(GTK_OBJECT(wbYes), "clicked",
                     GTK_SIGNAL_FUNC(DialogResponse), &iResponse[0]);
  gtk_box_pack_start(GTK_BOX(wBox2), wbYes, TRUE, TRUE, 0);
  gtk_widget_show(wbYes);
  
  wbNo = gtk_button_new_with_label("No");
  iResponse[1] = RESPONSE_NO;
  gtk_signal_connect(GTK_OBJECT(wbNo), "clicked",
                     GTK_SIGNAL_FUNC(DialogResponse), &iResponse[1]);
  gtk_box_pack_start(GTK_BOX(wBox2), wbNo, TRUE, TRUE, 0);
  gtk_widget_show(wbNo);
    
  /*
   *  Display the window
   */
  gtk_widget_show(wBox2);
  gtk_widget_show(wBox1);
  gtk_widget_show(wDialog);
  gtk_grab_add(wDialog);
  
  /*
   *  Disable the Guess button
   */
  gtk_label_set(GTK_LABEL(wButtonLabel), "");
  GTK_WIDGET_UNSET_FLAGS(wbGuess, GTK_CAN_FOCUS);
  gtk_widget_set_sensitive(wbGuess, FALSE);
  gtk_signal_disconnect(GTK_OBJECT(wbGuess), ButtonSignal);
  
  /*
   *  Wait for user response
   */
  bResponse = RESPONSE_NONE;
  wHighScoreName = NULL;
}


/*
 * Win Game dialog
 *   Congratulates and prompts for another game
 */
void WinGameDialog()
{
  GtkWidget	*wLabel,						/* text label					*/
  		*wBox1, *wBox2,					/* packing widgets				*/
  		*wbYes, *wbNo;					/* yes, no buttons				*/
  unsigned int	uiHours, uiMinutes, uiSeconds,	/* playing time					*/
  		uiTenths;
  char		sMessage[255];					/* text message					*/
  
  /*
   *  Compute the playing time
   */
  uiHours   = uiGameTime/36000;
  uiMinutes = (uiGameTime -uiHours*36000)/600;
  uiSeconds = (uiGameTime -uiHours*36000 -uiMinutes*600)/10;
  uiTenths  = uiGameTime % 10;

  
  wDialog = gtk_window_new(GTK_WINDOW_DIALOG);
  gtk_window_set_title(GTK_WINDOW(wDialog), "Game Over");
  gtk_window_position(GTK_WINDOW(wDialog), GTK_WIN_POS_CENTER);
  gtk_container_border_width(GTK_CONTAINER(wDialog), 10);
  
  wBox1 = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(wDialog), wBox1);
  
  /*
   *  Determine the end-of-game message
   */
  wLabel = gtk_label_new("Congratulations, you've won!");
  gtk_box_pack_start(GTK_BOX(wBox1), wLabel, FALSE, FALSE, 0);
  gtk_widget_show(wLabel);
  
  sprintf(sMessage, "Your time was %02u:%02u:%02u.%1u", uiHours, uiMinutes, uiSeconds,
          uiTenths);
  wLabel = gtk_label_new(sMessage);
  gtk_box_pack_start(GTK_BOX(wBox1), wLabel, FALSE, FALSE, 5);
  gtk_widget_show(wLabel);
  
  /*
   *  High score input, if necessary
   */
  if (CheckScore(iScore))
  {
    wLabel = gtk_label_new("You've made the top 10!\n"
                           "How shalt thee be called?");
    gtk_box_pack_start(GTK_BOX(wBox1), wLabel, FALSE, FALSE, 5);
    gtk_widget_show(wLabel);
    wHighScoreName = gtk_entry_new_with_max_length(32);
    gtk_box_pack_start(GTK_BOX(wBox1), wHighScoreName, FALSE, FALSE, 2);
    gtk_widget_show(wHighScoreName);
  }
  else
    wHighScoreName = NULL;
  
  /*
   *  Yes, No buttons
   */
  wLabel = gtk_label_new("Do you want to play again?");
  gtk_box_pack_start(GTK_BOX(wBox1), wLabel, FALSE, FALSE, 5);
  gtk_widget_show(wLabel);
  
  wBox2 = gtk_hbox_new(TRUE, 2);
  gtk_box_pack_end(GTK_BOX(wBox1), wBox2, FALSE, FALSE, 0);
  
  wbYes = gtk_button_new_with_label("Yes");
  iResponse[0] = RESPONSE_YES;
  gtk_signal_connect(GTK_OBJECT(wbYes), "clicked",
                     GTK_SIGNAL_FUNC(DialogResponse), &iResponse[0]);
  gtk_box_pack_start(GTK_BOX(wBox2), wbYes, TRUE, TRUE, 0);
  gtk_widget_show(wbYes);
  
  wbNo = gtk_button_new_with_label("No");
  iResponse[1] = RESPONSE_NO;
  gtk_signal_connect(GTK_OBJECT(wbNo), "clicked",
                     GTK_SIGNAL_FUNC(DialogResponse), &iResponse[1]);
  gtk_box_pack_start(GTK_BOX(wBox2), wbNo, TRUE, TRUE, 0);
  gtk_widget_show(wbNo);
    
  /*
   *  Display the window
   */
  gtk_widget_show(wBox2);
  gtk_widget_show(wBox1);
  gtk_widget_show(wDialog);
  gtk_grab_add(wDialog);
  
  /*
   *  Disable the Guess button
   */
  gtk_label_set(GTK_LABEL(wButtonLabel), "");
  GTK_WIDGET_UNSET_FLAGS(wbGuess, GTK_CAN_FOCUS);
  gtk_widget_set_sensitive(wbGuess, FALSE);
  gtk_signal_disconnect(GTK_OBJECT(wbGuess), ButtonSignal);
  
  /*
   *  Wait for user response
   */
  bResponse = RESPONSE_NONE;
}


/*
 * About dialog
 *   Displays version info
 */
void AboutDialog(gchar *sID)
{
  GtkWidget	*wLabel,						/* text label					*/
  		*wBox1,							/* packing widgets				*/
  		*wbClose;						/* close buttons				*/
  char		sVersion[255];					/* version string				*/

  /*
   *  Create the dialog window
   */
  wDialog = gtk_window_new(GTK_WINDOW_DIALOG);
  gtk_window_position(GTK_WINDOW(wDialog), GTK_WIN_POS_CENTER);
  gtk_window_set_title(GTK_WINDOW(wDialog), "About Stones of Asterind");
  gtk_container_border_width(GTK_CONTAINER(wDialog), 10);
  
  wBox1 = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(wDialog), wBox1);
  
  /*
   *  Display the message
   */
  sprintf(sVersion, "Stones of Asterind\nVersion %d.%d%s\n"
                    "(c) Copyright 1998 Matthew J. Gelhaus\n"
                    "All rights reserved.", MAJOR_VERSION, MINOR_VERSION,
                    POSTFIX_VERSION);
  wLabel = gtk_label_new(sVersion);
  gtk_box_pack_start(GTK_BOX(wBox1), wLabel, FALSE, FALSE, 0);
  gtk_widget_show(wLabel);
  
  wLabel = gtk_label_new("For my wonderful Taia Jo");
  gtk_box_pack_start(GTK_BOX(wBox1), wLabel, FALSE, FALSE, 20);
  gtk_widget_show(wLabel);

  /*
   *  Close button
   */
  wbClose = gtk_button_new_with_label("Close");
  iResponse[1] = RESPONSE_NO;
  gtk_signal_connect(GTK_OBJECT(wbClose), "clicked",
                     GTK_SIGNAL_FUNC(DialogResponse), &iResponse[1]);
  gtk_box_pack_start(GTK_BOX(wBox1), wbClose, TRUE, TRUE, 0);
  gtk_widget_show(wbClose);

  /*
   *  Display the window
   */
  gtk_widget_show(wBox1);
  gtk_widget_show(wDialog);
  gtk_grab_add(wDialog);
  
  /*
   *  Wait for user response
   */
  bResponse = RESPONSE_NONE;
}


/*
 * Game Options dialog
 *   Select game parameters
 */
void OptionsDialog(gchar *sID)
{
  GtkWidget	*wLabel,						/* text label					*/
  		*wBox1, *wBox2,					/* packing widgets				*/
  		*wOption, *wMenu, *wItem,		/* option menu widgets			*/
  		*wbRepeat,						/* repeat color checkbutton		*/
  		*wbNewGame, *wbOK, *wbCancel;	/* OK & Cancel buttons			*/
  int		iNumber;						/* loop counter					*/
  char		sMessage[255];

  /*
   *  Create the dialog window
   */
  wDialog = gtk_window_new(GTK_WINDOW_DIALOG);
  gtk_window_position(GTK_WINDOW(wDialog), GTK_WIN_POS_CENTER);
  gtk_window_set_title(GTK_WINDOW(wDialog), "Game Options");
  gtk_container_border_width(GTK_CONTAINER(wDialog), 10);
  
  wBox1 = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(wDialog), wBox1);
  
  /*
   *  Maximum number of tries
   */
  wBox2 = gtk_hbox_new(FALSE, 2);
  gtk_box_pack_start(GTK_BOX(wBox1), wBox2, FALSE, FALSE, 2);
  wLabel = gtk_label_new("Max. number of tries:");
  gtk_box_pack_start(GTK_BOX(wBox2), wLabel, FALSE, FALSE, 0);
  gtk_widget_show(wLabel);
  wOption = gtk_option_menu_new();
  wMenu = gtk_menu_new();
  for (iNumber = 1; iNumber <= ROWS; iNumber++)
  {
    sprintf(sMessage, "%d", iNumber);
    wItem = gtk_menu_item_new_with_label(sMessage);
    gtk_signal_connect(GTK_OBJECT(wItem), "activate", GTK_SIGNAL_FUNC(SetNumberRows),
                       (void *)iNumber);
    gtk_menu_append(GTK_MENU(wMenu), wItem);
    gtk_widget_show(wItem);
  }
  gtk_menu_set_active(GTK_MENU(wMenu), iNumRows -1);
  gtk_option_menu_set_menu(GTK_OPTION_MENU(wOption), wMenu);
  gtk_box_pack_start(GTK_BOX(wBox2), wOption, FALSE, FALSE, 0);
  gtk_widget_show(wOption);
  gtk_widget_show(wBox2);
  
  /*
   *  Numer of columns
   */
  wBox2 = gtk_hbox_new(FALSE, 2);
  gtk_box_pack_start(GTK_BOX(wBox1), wBox2, FALSE, FALSE, 2);
  wLabel = gtk_label_new("Number of columns:");
  gtk_box_pack_start(GTK_BOX(wBox2), wLabel, FALSE, FALSE, 0);
  gtk_widget_show(wLabel);
  wOption = gtk_option_menu_new();
  wMenu = gtk_menu_new();
  for (iNumber = 4; iNumber <= COLS; iNumber++)
  {
    sprintf(sMessage, "%d", iNumber);
    wItem = gtk_menu_item_new_with_label(sMessage);
    gtk_signal_connect(GTK_OBJECT(wItem), "activate", GTK_SIGNAL_FUNC(SetNumberCols),
                       (void *)iNumber);
    gtk_menu_append(GTK_MENU(wMenu), wItem);
    gtk_widget_show(wItem);
  }
  gtk_menu_set_active(GTK_MENU(wMenu), iNumCols -4);
  gtk_option_menu_set_menu(GTK_OPTION_MENU(wOption), wMenu);
  gtk_box_pack_start(GTK_BOX(wBox2), wOption, FALSE, FALSE, 0);
  gtk_widget_show(wOption);
  gtk_widget_show(wBox2);
  
  /*
   *  Number of available colors
   */
  wBox2 = gtk_hbox_new(FALSE, 2);
  gtk_box_pack_start(GTK_BOX(wBox1), wBox2, FALSE, FALSE, 2);
  wLabel = gtk_label_new("Available colors:");
  gtk_box_pack_start(GTK_BOX(wBox2), wLabel, FALSE, FALSE, 0);
  gtk_widget_show(wLabel);
  wOption = gtk_option_menu_new();
  wMenu = gtk_menu_new();
  for (iNumber = 2; iNumber <= COLORS; iNumber++)
  {
    sprintf(sMessage, "%d", iNumber);
    wItem = gtk_menu_item_new_with_label(sMessage);
    gtk_signal_connect(GTK_OBJECT(wItem), "activate", GTK_SIGNAL_FUNC(SetNumberColors),
                       (void *)iNumber);
    gtk_menu_append(GTK_MENU(wMenu), wItem);
    gtk_widget_show(wItem);
  }
  gtk_menu_set_active(GTK_MENU(wMenu), iNumColors -2);
  gtk_option_menu_set_menu(GTK_OPTION_MENU(wOption), wMenu);
  gtk_box_pack_start(GTK_BOX(wBox2), wOption, FALSE, FALSE, 0);
  gtk_widget_show(wOption);
  gtk_widget_show(wBox2);
  
  /*
   *  Allow repeat colors?
   */
  wBox2 = gtk_hbox_new(FALSE, 2);
  gtk_box_pack_start(GTK_BOX(wBox1), wBox2, FALSE, FALSE, 2);
  wbRepeat = gtk_check_button_new_with_label("Allow repeat colors");
  gtk_signal_connect(GTK_OBJECT(wbRepeat), "clicked",
                     GTK_SIGNAL_FUNC(SetRepeat), NULL);
  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(wbRepeat), bAllowRepeatColors);
  gtk_box_pack_start(GTK_BOX(wBox2), wbRepeat, FALSE, FALSE, 0);
  gtk_widget_show(wbRepeat);
  gtk_widget_show(wBox2);
  
  /*
   *  New Game, OK, Cancel buttons
   */
  wBox2 = gtk_hbox_new(TRUE, 2);
  gtk_box_pack_start(GTK_BOX(wBox1), wBox2, FALSE, FALSE, 5);
  wbNewGame = gtk_button_new_with_label("New game");
  iResponse[4] = RESPONSE_OK_NEW_GAME;
  gtk_signal_connect(GTK_OBJECT(wbNewGame), "clicked",
                     GTK_SIGNAL_FUNC(DialogResponse), &iResponse[4]);
  gtk_box_pack_start(GTK_BOX(wBox2), wbNewGame, TRUE, TRUE, 0);
  gtk_widget_show(wbNewGame);
  wbOK = gtk_button_new_with_label("Ok");
  iResponse[2] = RESPONSE_OK;
  gtk_signal_connect(GTK_OBJECT(wbOK), "clicked",
                     GTK_SIGNAL_FUNC(DialogResponse), &iResponse[2]);
  gtk_box_pack_start(GTK_BOX(wBox2), wbOK, TRUE, TRUE, 0);
  gtk_widget_show(wbOK);
  wbCancel = gtk_button_new_with_label("Cancel");
  iResponse[3] = RESPONSE_CANCEL;
  gtk_signal_connect(GTK_OBJECT(wbCancel), "clicked",
                     GTK_SIGNAL_FUNC(DialogResponse), &iResponse[3]);
  gtk_box_pack_start(GTK_BOX(wBox2), wbCancel, TRUE, TRUE, 0);
  gtk_widget_show(wbCancel);
  gtk_widget_show(wBox2);

  /*
   *  Display the window
   */
  gtk_widget_show(wBox1);
  gtk_widget_show(wDialog);
  gtk_grab_add(wDialog);
  
  /*
   * Initialize the Options struct
   */
  Options.iRows = iNumRows;
  Options.iCols = iNumCols;
  Options.iColors = iNumColors;
  
  /*
   *  Wait for user response
   */
  bResponse = RESPONSE_NONE;
}


/*
 * High scores dialog
 *   Show the best of the best!
 */
void HighScoresDialog(gchar *sID)
{
  GtkWidget	*wScoresList;				/* high-scores list					*/
  GtkWidget	*wBox1;						/* layout box						*/
  gchar		*sTitles[8] = {"Score", "Name", "Time", "# Tries", "Colors",
                               "Columns", "Rows", "Date"};
  gchar		*sEntry[8];
  GtkWidget	*wbClose;					/* close button						*/
  int		iColumn, iScore;			/* loop counters					*/
  int		iGameScore, iTries, iNumRows, iNumCols, iNumColors;
  unsigned int	uiDate, uiTime;
  unsigned int	uiHour, uiMin, uiSec, uiTenths;
  
  /*
   *  Allocate space for the entry array
   */
  for (iColumn = 0; iColumn < 8; iColumn++)
  {
    sEntry[iColumn] = (gchar *)malloc(64*sizeof(gchar));
    sprintf(sEntry[iColumn], "Test");
  }
  
  /*
   *  Create the dialog window
   */
  wDialog = gtk_window_new(GTK_WINDOW_DIALOG);
  gtk_window_position(GTK_WINDOW(wDialog), GTK_WIN_POS_CENTER);
  gtk_window_set_title(GTK_WINDOW(wDialog), "Hall of Fame");
  gtk_container_border_width(GTK_CONTAINER(wDialog), 10);
  gtk_widget_set_usize(wDialog, 534, 248);
  
  /*
   *  Layout box
   */
  wBox1 = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(wDialog), wBox1);
  
  /*
   *  Create the high-scores list display
   */
  wScoresList = gtk_clist_new_with_titles(8, sTitles);
  gtk_clist_set_selection_mode(GTK_CLIST(wScoresList), GTK_SELECTION_BROWSE);
#ifndef __hpux
  gtk_clist_set_policy(GTK_CLIST(wScoresList), GTK_POLICY_AUTOMATIC,
                       GTK_POLICY_AUTOMATIC);
  gtk_clist_set_border(GTK_CLIST(wScoresList), GTK_SHADOW_ETCHED_OUT);
#endif
  for (iColumn = 0; iColumn < 8; iColumn++)
    gtk_clist_set_column_justification(GTK_CLIST(wScoresList), iColumn,
                                       GTK_JUSTIFY_CENTER);
  gtk_clist_set_column_width(GTK_CLIST(wScoresList), 0, 48);
  gtk_clist_set_column_width(GTK_CLIST(wScoresList), 1, 128);
  gtk_clist_set_column_width(GTK_CLIST(wScoresList), 2, 64);
  gtk_clist_set_column_width(GTK_CLIST(wScoresList), 3, 48);
  gtk_clist_set_column_width(GTK_CLIST(wScoresList), 4, 48);
  gtk_clist_set_column_width(GTK_CLIST(wScoresList), 5, 48);
  gtk_clist_set_column_width(GTK_CLIST(wScoresList), 6, 32);
  gtk_clist_set_column_width(GTK_CLIST(wScoresList), 7, 32);
  gtk_box_pack_start(GTK_BOX(wBox1), wScoresList, TRUE, TRUE, 2);
  gtk_widget_show(wScoresList);
  
  /*
   *  Load the current high-scores list
   */
  for (iScore = 0; iScore < iNumScores; iScore++)
  {
    GetHighScore(iScore, &iGameScore, sEntry[1], &uiTime, &uiDate, &iTries,
                 &iNumRows, &iNumCols, &iNumColors);
    uiHour   = uiTime/36000;
    uiMin    = (uiTime -uiHour*36000)/600;
    uiSec    = (uiTime -uiHour*36000 -uiMin*600)/10;
    uiTenths = uiTime % 10;
    sprintf(sEntry[0], "%d", iGameScore);
    sprintf(sEntry[2], "%02u:%02u:%02u.%u", uiHour, uiMin, uiSec, uiTenths);
    sprintf(sEntry[3], "%d", iTries);
    sprintf(sEntry[4], "%d", iNumColors);
    sprintf(sEntry[5], "%d", iNumCols);
    sprintf(sEntry[6], "%d", iNumRows);
    sprintf(sEntry[7], "%u", uiDate);
    gtk_clist_append(GTK_CLIST(wScoresList), sEntry);
  }
  
  /*
   *  Close button
   */
  wbClose = gtk_button_new_with_label("Close");
  iResponse[1] = RESPONSE_NO;
  gtk_signal_connect(GTK_OBJECT(wbClose), "clicked",
                     GTK_SIGNAL_FUNC(DialogResponse), &iResponse[1]);
  gtk_box_pack_start(GTK_BOX(wBox1), wbClose, FALSE, FALSE, 2);
  gtk_widget_show(wbClose);

  /*
   *  Display the window
   */
  gtk_widget_show(wBox1);
  gtk_widget_show(wDialog);
  gtk_grab_add(wDialog);
  
  /*
   *  Free allocated space
   */
  for (iColumn = 0; iColumn < 8; iColumn++)
    free(sEntry[iColumn]);
  
  
  /*
   *  Wait for user response
   */
  bResponse = RESPONSE_NONE;
}


void SetNumberRows(GtkWidget *wItem, gpointer pData)
{
  Options.iRows = (int)pData;
}


void SetNumberCols(GtkWidget *wItem, gpointer pData)
{
  Options.iCols = (int)pData;
}


void SetNumberColors(GtkWidget *wItem, gpointer pData)
{
  Options.iColors = (int)pData;
}


void SetRepeat(GtkWidget *wButton, gpointer pData)
{
  if (GTK_TOGGLE_BUTTON(wButton)->active)
    Options.bAllowRepeatColors = TRUE;
  else
    Options.bAllowRepeatColors = FALSE;
}


/*
 * Low Color Warning dialog
 *   Displays warning about minimum # colors required for no-repeat option
 */
void LowColorWarningDialog(gchar *sID)
{
  GtkWidget	*wLabel,						/* text label					*/
  		*wBox1,							/* packing widgets				*/
  		*wbClose;						/* close buttons				*/

  /*
   *  Create the dialog window
   */
  wDialog = gtk_window_new(GTK_WINDOW_DIALOG);
  gtk_window_position(GTK_WINDOW(wDialog), GTK_WIN_POS_CENTER);
  gtk_window_set_title(GTK_WINDOW(wDialog), "WARNING");
  gtk_container_border_width(GTK_CONTAINER(wDialog), 10);
  
  wBox1 = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(wDialog), wBox1);
  
  /*
   *  Display the message
   */
  wLabel = gtk_label_new("In order to satisfy the no-repeat\n"
                         "colors option, the number of colors\n"
                         "has been raised to equal the number\n"
                         "of columns.");
  gtk_box_pack_start(GTK_BOX(wBox1), wLabel, FALSE, FALSE, 0);
  gtk_widget_show(wLabel);
  
  /*
   *  Close button
   */
  wbClose = gtk_button_new_with_label("Close");
  iResponse[1] = RESPONSE_NO;
  gtk_signal_connect(GTK_OBJECT(wbClose), "clicked",
                     GTK_SIGNAL_FUNC(DialogResponse), &iResponse[1]);
  gtk_box_pack_start(GTK_BOX(wBox1), wbClose, TRUE, TRUE, 0);
  gtk_widget_show(wbClose);

  /*
   *  Display the window
   */
  gtk_widget_show(wBox1);
  gtk_widget_show(wDialog);
  gtk_grab_add(wDialog);
  
  /*
   *  Wait for user response
   */
  bResponse = RESPONSE_NONE;
}
