/*  gameboard.c -	Defines the gameboard GUI
 *
 *  Stones of Asterind
 *
 *  Matthew J. Gelhaus
 *  October 24, 1998
 */
 
#include <sys/time.h>
#include <unistd.h>
#include <gtk/gtk.h>

#include "gameboard.h"
#include "globals.h"
#include "gtkled.h"
#include "gtkclock.h"

/*
 *  Create the game board
 */
GtkWidget *CreateGameBoard(int iNumRows, int iNumCols)
{
  GtkWidget	*wGrid;					/* game board grid				*/
  int		iRow, iCol;				/* row, column indices			*/
  
  /*
   *  Game board layout is via table
   */
  wGrid = gtk_table_new(iNumRows, iNumCols*3 +1, TRUE);
  gtk_table_set_row_spacings(GTK_TABLE(wGrid), 3);
  
  /*
   *  Fill in the table cells
   */
  for (iRow = 0; iRow < iNumRows; iRow++)
    for (iCol = 0; iCol < iNumCols; iCol++)
    {
      /* Hint pegs */
      wHint[iRow][iCol] = gtk_led_new();
      gtk_table_attach(GTK_TABLE(wGrid), wHint[iRow][iCol],
                       iCol, iCol +1, iRow, iRow +1,
                       GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND,
                       1, 0);
      gtk_led_set_color(GTK_LED(wHint[iRow][iCol]), 49152, 49152, 49152);
      gtk_widget_show(wHint[iRow][iCol]);
      
      /* Player pegs */
      wPeg[iRow][iCol] = gtk_button_new();
      gtk_table_attach_defaults(GTK_TABLE(wGrid), wPeg[iRow][iCol],
                                iCol*2 +iNumCols +1, iCol*2 +iNumCols +3,
                                iRow, iRow +1);
      GTK_WIDGET_UNSET_FLAGS(wPeg[iRow][iCol], GTK_CAN_FOCUS);
      gtk_widget_set_sensitive(wPeg[iRow][iCol], FALSE);
      iCoord[iRow][iCol][0] = iRow;
      iCoord[iRow][iCol][1] = iCol;
      gtk_signal_connect(GTK_OBJECT(wPeg[iRow][iCol]), "clicked",
                         GTK_SIGNAL_FUNC(ChangeColor),
                         (gpointer)iCoord[iRow][iCol]);
      gtk_widget_show(wPeg[iRow][iCol]);
      
      /* Player peg LEDs */
      wPegLED[iRow][iCol] = gtk_led_new();
      gtk_container_add(GTK_CONTAINER(wPeg[iRow][iCol]), wPegLED[iRow][iCol]);
      gtk_led_set_color(GTK_LED(wPegLED[iRow][iCol]), 49152, 49152, 49152);
      gtk_widget_show(wPegLED[iRow][iCol]);
    }

  /*
   *  Return a pointer to the board
   */
  return wGrid;
}




/*
 *  Enable the specified row.
 *    NOTE:  iRow parameter is 1..rows
 */
void EnableRow(int iRow)
{
  int		iCol;					/* column index					*/

  for (iCol = 0; iCol < iNumCols; iCol++)
  {
    GTK_WIDGET_SET_FLAGS(wPeg[iNumRows -iRow][iCol], GTK_CAN_FOCUS);
    gtk_widget_set_sensitive(wPeg[iNumRows -iRow][iCol], TRUE);
    SetButtonColor(wPegLED[iNumRows -iRow][iCol], iPegColor[iCol]);
  }
}
  

/*
 *  Disable the specified row.
 *    NOTE:  iRow parameter is 1..rows
 */
void DisableRow(int iRow)
{
  int		iCol;					/* column index					*/
  
  for (iCol = 0; iCol < iNumCols; iCol++)
  {
    GTK_WIDGET_UNSET_FLAGS(wPeg[iNumRows -iRow][iCol], GTK_CAN_FOCUS);
    gtk_widget_set_sensitive(wPeg[iNumRows -iRow][iCol], FALSE);
  }
}


/*
 *  Peg callback - change the button's color
 */
void ChangeColor(GtkWidget *wButton, gpointer pData)
{
  int iRow, iCol;

  /*
   *  Extract the row and column
   */
  iRow = ((int *)pData)[0];
  iCol = ((int *)pData)[1];
  
  /*
   *  Increment the color index
   */
  if (++iPegColor[iCol] > (iNumColors -1))
    iPegColor[iCol] = 0;
  
  SetButtonColor(wPegLED[iRow][iCol], iPegColor[iCol]);
}
  

/*
 *  Update the game timer
 */
gint UpdateTime(gpointer pData)
{
  struct timeval tvTimeVal;						/* used by game timer			*/
  struct timezone tzTimeZone;					/* used by game timer			*/
  
  gettimeofday(&tvTimeVal, &tzTimeZone);

  /*
   *  Compute the time elapsed
   */
  uiGameTime = (unsigned int)(tvTimeVal.tv_sec*10 +tvTimeVal.tv_usec/100000) -uiBaseTime;

  /*
   *  Update the clock display
   */
  gtk_clock_set_time(GTK_CLOCK(wClock), uiGameTime);
 
  /*
   *  Keep calling this function
   */ 
  return TRUE;
}


/*
 *  Set a button color
 */
void SetButtonColor(GtkWidget *wLED, int iColor)
{
  guint16	uiRed, uiGreen, uiBlue;	/* RGB components of LED color	*/

  /*
   *  Determine the new color components
   */
  switch (iColor)
  {
    case 0:  uiRed =     0;  uiGreen =     0;  uiBlue =     0;  break;
    case 1:  uiRed = 65535;  uiGreen =     0;  uiBlue =     0;  break;
    case 2:  uiRed = 65535;  uiGreen = 65535;  uiBlue =     0;  break;
    case 3:  uiRed =     0;  uiGreen = 65535;  uiBlue =     0;  break;
    case 4:  uiRed =     0;  uiGreen = 65535;  uiBlue = 65535;  break;
    case 5:  uiRed =     0;  uiGreen =     0;  uiBlue = 65535;  break;
    case 6:  uiRed = 65535;  uiGreen =     0;  uiBlue = 65535;  break;
    case 7:  uiRed = 65535;  uiGreen = 65535;  uiBlue = 65535;  break;
    case 8:  uiRed = 49152;  uiGreen = 49152;  uiBlue =     0;  break;
    case 9:  uiRed = 32768;  uiGreen = 32768;  uiBlue = 32768;  break;
    case 10: uiRed = 65535;  uiGreen = 16384;  uiBlue = 16384;  break;
    case 11: uiRed = 32768;  uiGreen = 49152;  uiBlue = 32768;  break;
    default: uiRed = 32768;  uiGreen = 32768;  uiBlue = 32768;  break;
  }

  /*
   *  Set the LED color
   */
  gtk_led_set_color(GTK_LED(wLED), uiRed, uiGreen, uiBlue);  
}
