/*  hiscores.c -	manages High Scores list
 *
 *  Stones of Asterind
 *
 *  Matthew J. Gelhaus
 *  October 2, 1998
 */
 
#include <stdio.h>
#include <string.h>
#include "hiscores.h"

#define MAXSCORES			10

struct _HighScore
{
  int		iScore;
  char		sName[32];
  unsigned int 	uiTime,
  		uiDate;
  int		iRows,
	  	iNumRows,
 	 	iNumCols,
  		iNumColors;
};


typedef struct _HighScore HighScore;

HighScore	Score[MAXSCORES];


/*
 *  Check to see if a score qualifies for the high-score list
 */
int CheckScore(int iScore)
{
  int iIndex;
  
  if ((iNumScores == 0) || (iNumScores < MAXSCORES)) return TRUE;
  
  for (iIndex = 0; iIndex < iNumScores; iIndex++)
    if (iScore > Score[iIndex].iScore)
      return TRUE;
  
  return FALSE;
}


/*
 *  Add a new score to the high-score list
 */
void NewHighScore(int iScore, const char *sName, unsigned int uiTime, 
                  unsigned int uiDate, int iRows, int iNumRows,
                  int iNumCols, int iNumColors)
{
  int iIndex, iNewPosition;
  
  /*
   *  Find the appropriate position in the list
   */
  for (iIndex = 0; (iScore < Score[iIndex].iScore) && (iIndex < iNumScores);
       iIndex++);
  iNewPosition = iIndex;
  
  /*
   *  Abort if the new score doesn't qualify
   */
  if (iNewPosition == MAXSCORES) return;
  
  /*
   *  Shift the lower entries down to make room
   */
  for (iIndex = (iNumScores < MAXSCORES ? iNumScores : MAXSCORES -1);
       iIndex > iNewPosition; iIndex--)
    Score[iIndex +1] = Score[iIndex];
  
  /*
   *  Insert the new entry
   */
  Score[iNewPosition].iScore = iScore;
  strcpy(Score[iNewPosition].sName, sName);
  Score[iNewPosition].uiTime = uiTime;
  Score[iNewPosition].uiDate = uiDate;
  Score[iNewPosition].iRows = iRows;
  Score[iNewPosition].iNumRows = iNumRows;
  Score[iNewPosition].iNumCols = iNumCols;
  Score[iNewPosition].iNumColors = iNumColors;
  
  /*
   *  Increment the number of scores, if necessary
   */
  if (iNumScores < MAXSCORES) iNumScores++;
}
    

void GetHighScore(int iRank, int *iScore, char *sName, unsigned int *uiTime,
                  unsigned int *uiDate, int *iRows, int *iNumRows,
                  int *iNumCols, int *iNumColors)
{
  if (iRank > iNumScores) return;

  *iScore = Score[iRank].iScore;
  strcpy(sName, Score[iRank].sName);
  *uiTime = Score[iRank].uiTime;
  *uiDate = Score[iRank].uiDate;
  *iRows = Score[iRank].iRows;
  *iNumRows = Score[iRank].iNumRows;
  *iNumCols = Score[iRank].iNumCols;
  *iNumColors = Score[iRank].iNumColors;
}


void SaveHighScores()
{
  FILE *OutFile;
  int iIndex;
  int iChar;
  
  OutFile = fopen(".sa_hiscores", "w");
  if (OutFile != NULL)
  {
    fprintf(OutFile, "%d\n", iNumScores);
    for (iIndex = 0; iIndex < iNumScores; iIndex++)
    {
      fprintf(OutFile, "%d,", Score[iIndex].iScore);
      for (iChar = 0; iChar < 32; iChar++)
        fprintf(OutFile, "%02X", Score[iIndex].sName[iChar]);
      fprintf(OutFile, ",%u,%u,%d,%d,%d,%d\n", 
              Score[iIndex].uiTime, Score[iIndex].uiDate,
              Score[iIndex].iRows, Score[iIndex].iNumRows,
              Score[iIndex].iNumCols, Score[iIndex].iNumColors);
    }
    fclose(OutFile);
  }
}


void LoadHighScores()
{
  FILE *InFile;
  int iIndex;
  int iChar;
  
  InFile = fopen(".sa_hiscores", "r");
  if (InFile != NULL)
  {
    fscanf(InFile, "%d", &iNumScores);
    for (iIndex = 0; iIndex < iNumScores; iIndex++)
    {
      fscanf(InFile, "%d,", &(Score[iIndex].iScore));
      for (iChar = 0; iChar < 32; iChar++)
        fscanf(InFile, "%02X", &(Score[iIndex].sName[iChar]));
      fscanf(InFile, ",%u,%u,%d,%d,%d,%d\n", 
             &(Score[iIndex].uiTime),
             &(Score[iIndex].uiDate), &(Score[iIndex].iRows),
             &(Score[iIndex].iNumRows), &(Score[iIndex].iNumCols),
             &(Score[iIndex].iNumColors));
    }
    fclose(InFile);
  }
  else
    iNumScores = 0;
}
