/*  menu.c -	Main menu functions
 *
 *  Stones of Asterind
 *
 *  Matthew J. Gelhaus
 *  October 24, 1998
 */
 
#include <gtk/gtk.h>

#include "main.h"
#include "menu.h"
#include "dialogs.h"
#include "globals.h"
#include "gameboard.h"
#include "newgame.h"

/*
 *  Create the main menu
 */
GtkWidget *CreateMainMenu()
{
  GtkWidget	*wMainMenu,				/* main menu bar				*/
  		*wGameItem,				/* game menu item				*/
  		*wHelpItem,				/* help menu item				*/
  		*wGameMenu,				/* game menu					*/
  		*wHelpMenu,				/* help menu					*/
  		*wGameNewItem,			/* game, new menu item			*/
  		*wGameOptionsItem,		/* game, preferences menu item	*/
  		*wGameHighscoresItem,	/* game, highscores menu item	*/
  		*wGameExitItem,			/* game, exit menu item			*/
  		*wHelpAboutItem,		/* help, about menu item		*/
  		*wMenuSeparator;		/* menu separator				*/

  /*
   *  'Game' menu
   */
  wGameMenu = gtk_menu_new();
  
  wGameNewItem = gtk_menu_item_new_with_label("New game");
  wGameOptionsItem = gtk_menu_item_new_with_label("Options...");
  wGameHighscoresItem = gtk_menu_item_new_with_label("High scores...");
  wMenuSeparator = gtk_menu_item_new();
  wGameExitItem = gtk_menu_item_new_with_label("Exit");
  
  gtk_menu_append(GTK_MENU(wGameMenu), wGameNewItem);
  gtk_menu_append(GTK_MENU(wGameMenu), wGameOptionsItem);
  gtk_menu_append(GTK_MENU(wGameMenu), wGameHighscoresItem);
  gtk_menu_append(GTK_MENU(wGameMenu), wMenuSeparator);
  gtk_menu_append(GTK_MENU(wGameMenu), wGameExitItem);
  
  gtk_signal_connect_object(GTK_OBJECT(wGameNewItem), "activate",
                            GTK_SIGNAL_FUNC(NewGame), (gpointer) "game.new");
  gtk_signal_connect_object(GTK_OBJECT(wGameOptionsItem), "activate",
                            GTK_SIGNAL_FUNC(OptionsDialog), 
                            (gpointer)"game.options");
  gtk_signal_connect_object(GTK_OBJECT(wGameHighscoresItem), "activate",
                            GTK_SIGNAL_FUNC(HighScoresDialog),
                            (gpointer)"game.highscores");
  gtk_signal_connect(GTK_OBJECT(wGameExitItem), "activate",
                     GTK_SIGNAL_FUNC(ExitGame), (gpointer) "game.exit");
  
  gtk_widget_show(wGameNewItem);
  gtk_widget_show(wGameOptionsItem);
  gtk_widget_show(wGameHighscoresItem);
  gtk_widget_show(wMenuSeparator);
  gtk_widget_show(wGameExitItem);
  
  /*
   *  'Help' menu
   */
  wHelpMenu = gtk_menu_new();
  
  wHelpAboutItem = gtk_menu_item_new_with_label("About Stones of Asterind...");
  
  gtk_menu_append(GTK_MENU(wHelpMenu), wHelpAboutItem);
  
  gtk_signal_connect_object(GTK_OBJECT(wHelpAboutItem), "activate",
                            GTK_SIGNAL_FUNC(AboutDialog), (gpointer) "help.about");

  gtk_widget_show(wHelpAboutItem);
  
  /*
   *  Main menu
   */
  wMainMenu = gtk_menu_bar_new();
  
  wGameItem = gtk_menu_item_new_with_label("Game");
  wHelpItem = gtk_menu_item_new_with_label("Help");
  
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(wGameItem), wGameMenu);
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(wHelpItem), wHelpMenu);
  
  gtk_menu_bar_append(GTK_MENU_BAR(wMainMenu), wGameItem);
  gtk_menu_bar_append(GTK_MENU_BAR(wMainMenu), wHelpItem);
  
  gtk_widget_show(wGameItem);
  gtk_widget_show(wHelpItem);

  /*
   *  Return a pointer to the new menu
   */
  return wMainMenu;
}


/*
 * Create a new game
 */
void NewGame(gchar *sID)
{
  /*
   *  Stop the clock if necessary
   */
  gtk_timeout_remove(GameTimer);

  /*
   *  Delete the old game board
   */
  gtk_widget_destroy(wGameBoard);

  /*
   *  Setup a new game
   */
  wGameBoard = InitGame(iNumRows, iNumCols, &iNumColors);
  gtk_box_pack_end(GTK_BOX(wBox1), wGameBoard, TRUE, TRUE, 2);
  gtk_widget_show(wGameBoard);
}

