/*  gameboard.c -	Defines the gameboard GUI
 *
 *  Stones of Asterind
 *
 *  Matthew J. Gelhaus
 *  October 24, 1998
 */

#include <sys/time.h>
#include <unistd.h>
#include <gtk/gtk.h>

#include "newgame.h"
#include "main.h"
#include "gameboard.h"
#include "dialogs.h"
#include "globals.h"
#include "gtkled.h"
#include "gtkclock.h"

#ifdef __hpux
#define RAND_MAX 255
#endif

GtkWidget *InitGame(int iNumRows, int iNumCols, int *iNumColors)
{
  GtkWidget	*wGrid;					/* game board grid				*/
  GtkWidget	*wBox2, *wBox3;			/* layout boxes					*/
  GtkWidget	*wFrame;				/* frame						*/
  int		iRow;					/* row index					*/
  int		iCol;					/* column index					*/
  GtkWidget	*wBox4;					/* layout box					*/
  
  /*
   *  Check for enough colors and adjust if necessary
   */
  if ((bAllowRepeatColors == FALSE) && (*iNumColors < iNumCols))
  {
    *iNumColors = iNumCols;
    LowColorWarningDialog();
  }
  
  /*
   *  Make up a random solution
   */
  InitSolution(iSolution, iNumCols, *iNumColors, bAllowRepeatColors);
  
  /*
   *  Game board layout boxes
   */
  wBox2 = gtk_hbox_new(FALSE, 0);
  
  wBox3 = gtk_vbox_new(FALSE, 0);
  gtk_box_pack_end(GTK_BOX(wBox2), wBox3, FALSE, FALSE, 2);
  gtk_widget_show(wBox3);
  
  /*
   *  Create the game board
   */
  wGrid = CreateGameBoard(iNumRows, iNumCols);
  gtk_box_pack_start(GTK_BOX(wBox2), wGrid, TRUE, TRUE, 2);
  gtk_widget_show(wGrid);
  
  /*
   *  Set the initial peg colors
   */
  for (iCol = 0; iCol < iNumCols; iCol++)
    iPegColor[iCol] = 0;
  
  /*
   *  Create the 'Guess' button
   */
  wbGuess = gtk_button_new();
  wButtonLabel = gtk_label_new("Start!");
  gtk_container_add(GTK_CONTAINER(wbGuess), wButtonLabel);
  gtk_widget_show(wButtonLabel);
  ButtonSignal = gtk_signal_connect(GTK_OBJECT(wbGuess), "clicked",
                                    GTK_SIGNAL_FUNC(StartGame), 
                                    &iCurrentRow);
  gtk_box_pack_end(GTK_BOX(wBox3), wbGuess, FALSE, FALSE, 0);
  gtk_widget_show(wbGuess);
  
  /*
   *  Create the score widget
   */
  wBox4 = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(wBox3), wBox4, FALSE, FALSE, 0);
  wScore = gtk_label_new("Score:");
  gtk_box_pack_start(GTK_BOX(wBox4), wScore, FALSE, FALSE, 0);
  gtk_widget_show(wScore);
  wScore = gtk_label_new("0");
  gtk_label_set_justify(GTK_LABEL(wScore), GTK_JUSTIFY_RIGHT);
  gtk_box_pack_end(GTK_BOX(wBox4), wScore, FALSE, FALSE, 0);
  gtk_widget_show(wScore);
  gtk_widget_show(wBox4);
  
  /*
   *  Create the game timer widget
   */
  wFrame = gtk_frame_new(NULL);
  gtk_frame_set_shadow_type(GTK_FRAME(wFrame), GTK_SHADOW_IN);
  gtk_box_pack_start(GTK_BOX(wBox3), wFrame, FALSE, FALSE, 0);
  gtk_widget_show(wFrame);
  wClock = gtk_clock_new();
  gtk_widget_set_usize(wClock, 100, 100);
  gtk_container_add(GTK_CONTAINER(wFrame), wClock);
  gtk_clock_set_time(GTK_CLOCK(wClock), 0);
  gtk_widget_show(wClock);

  /*
   *  Make a color key
   */
  wFrame = gtk_frame_new(NULL);
  gtk_frame_set_shadow_type(GTK_FRAME(wFrame), GTK_SHADOW_IN);
  gtk_box_pack_start(GTK_BOX(wBox3), wFrame, FALSE, FALSE, 0);
  gtk_widget_show(wFrame);
  wGrid = gtk_table_new(3, 4, TRUE);
  for (iRow = 0; iRow < 3; iRow++)
    for (iCol = 0; iCol < 4; iCol++)
    {
      wKey[iRow*4 +iCol] = gtk_led_new();
      gtk_table_attach(GTK_TABLE(wGrid), wKey[iRow*4 +iCol],
                       iCol, iCol +1, iRow, iRow +1,
                       GTK_FILL | GTK_EXPAND | GTK_SHRINK, 
                       GTK_FILL | GTK_EXPAND | GTK_SHRINK,
                       0, 0);
      SetButtonColor(wKey[iRow*4 +iCol], iRow*4 +iCol);
      if (iRow*4 +iCol < *iNumColors)
        gtk_widget_show(wKey[iRow*4 +iCol]);
    }
  gtk_widget_set_usize(wGrid, 100, 70);
  gtk_container_add(GTK_CONTAINER(wFrame), wGrid);
  gtk_widget_show(wGrid);

  /*
   *  Set the initial game parameters
   */
  iCurrentRow = 0;
  GameTimer = 0;
  iScore = 0;

  /*
   *  Return the widget pointer to the entire game board
   */
  return wBox2;
}


/*
 *  Initialize the solution
 */
void InitSolution(int *iSolution, int iColumns, int iNumColors,
                  int bRepeatColors)
{
  int iCol;							/* column index						*/
  int iSolutionColor;					/* random solution color			*/
  int *iColor;							/* list of colors					*/
  int iFactor;							/* rand() computation factor		*/
  
  iFactor = RAND_MAX/iNumColors;
  
  /*
   *  Procedure for non-repeating color solutions
   */
  if (!bRepeatColors)
  {
    /*
     *  Setup color list if we can't repeat colors in the solution
     */
    iColor = (int *)malloc((iNumColors +1)*sizeof(int));
    for (iCol = 0; iCol < iNumColors; iCol++)
      iColor[iCol] = FALSE;
    iColor[iNumColors] = TRUE;
    
    /*
     *  Compute the random colors, checking for repeats
     */
    for (iCol = 0; iCol < iColumns; iCol++)
    {
      iSolutionColor = iNumColors;
      while (iColor[iSolutionColor])
      {
        iSolutionColor = rand()/iFactor;
        if (iSolutionColor >= iNumColors)
          iSolutionColor = iNumColors -1;
      }
      
      /*
       *  Mark the color so we don't use it again
       */
      iColor[iSolutionColor] = TRUE;
      
      /*
       *  Assign the new color
       */
      iSolution[iCol] = iSolutionColor;
    }

    /*
     *  Free the color list
     */
    free(iColor);
  }
  
  /*
   *  Procedure for repeating color solutions
   */
  else
  {
    /*
     *  Compute the random colors
     */
    for (iCol = 0; iCol < iColumns; iCol++)
    {
      iSolutionColor = rand()/iFactor;
      if (iSolutionColor >= iNumColors)
        iSolutionColor = iNumColors -1;

      /*
       *  Assign the new color
       */
      iSolution[iCol] = iSolutionColor;
    }
  }
}  

    
/*
 *  Initialize the peg colors
 */
void InitColors(void)
{
  int		iCol;					/* column index					*/

  /*
   *  Set the initial peg colors
   */
   for (iCol = 0; iCol < ROWS; iCol++)
    iPegColor[iCol] = 0;
}


/*
 *  Start! button callback - begin the game
 */
void StartGame(GtkWidget *wButton, gpointer pData)
{
  struct timeval tvTimeVal;				/* used by game timer			*/
  struct timezone tzTimeZone;			/* used by game timer			*/

  /*
   *  Setup the game board for 1st play
   */
  EnableRow(iCurrentRow +1);

  /*
   * Start the game clock
   */
  GameTimer = gtk_timeout_add(200, UpdateTime, NULL);
  gettimeofday(&tvTimeVal, &tzTimeZone);
  uiBaseTime = (unsigned int)(tvTimeVal.tv_sec*10 +tvTimeVal.tv_usec/100000);
  	
  /*
   *  Change the label on the button and reconnect to new signal
   */
  gtk_signal_disconnect(GTK_OBJECT(wbGuess), ButtonSignal);
  gtk_label_set(GTK_LABEL(wButtonLabel), "Guess");
  ButtonSignal = gtk_signal_connect(GTK_OBJECT(wbGuess), "clicked",
                                    GTK_SIGNAL_FUNC(Guess), 
                                    &iCurrentRow);
}

